@@ -1,31 +1,33 @@
 mod cli;
-mod error;
 mod subcommands;
 
+use anyhow::{bail, Result};
 use clap::StructOpt;
 use cli::{Ferium, ProfileSubCommands, SubCommands};
-use error::{Error, Result};
+use colored::{ColoredString, Colorize};
 use ferinth::Ferinth;
 use furse::Furse;
-use libium::config;
+use lazy_static::lazy_static;
+use libium::{add, config};
 use tokio::{
 	fs::{create_dir_all, remove_dir_all},
 	io::AsyncReadExt,
 };
 
+lazy_static! {
+	pub static ref TICK: ColoredString = "✓".green();
+}
+
 #[tokio::main]
 async fn main() {
 	if let Err(err) = actual_main().await {
-		// If an error occures, print the error message
-		println!("{}", err);
-		// And exit with an exit code
+		eprintln!("{}", err.to_string().red().bold());
 		std::process::exit(1);
 	}
 }
 
 async fn actual_main() -> Result<()> {
-	// Get the command to execute from Clap
-	// This also displays the help page or version
+	// This also displays the help page or version automatically
 	let cli_app = Ferium::parse();
 
 	// Check for an internet connection
@@ -35,12 +37,8 @@ async fn actual_main() -> Result<()> {
 		// and check for 4 more seconds
 		eprint!("Checking internet connection... ");
 		match online::check(Some(4)).await {
-			Ok(_) => println!("✓"),
-			Err(_) => {
-				return Err(Error::Quit(
-					"× Ferium requires an internet connection to work",
-				))
-			},
+			Ok(_) => println!("{}", *TICK),
+			Err(_) => bail!("× Ferium requires an internet connection to work"),
 		}
 	};
 
@@ -57,22 +55,7 @@ async fn actual_main() -> Result<()> {
 	config_file
 		.read_to_string(&mut config_file_contents)
 		.await?;
-	// Deserialise `config_file` to a config
-	let mut config: config::structs::Config = match serde_json::from_str(&config_file_contents) {
-		Ok(config) => config,
-		Err(err) => {
-			return Err(Error::QuitFormatted(format!(
-				"Error decoding configuration file, {} at {:?} {}:{}",
-				// Error name
-				Error::JSONError(err.classify()),
-				// File path so that users can find it
-				config::config_file_path(),
-				// Location within config file
-				err.line(),
-				err.column()
-			)));
-		},
-	};
+	let mut config: config::structs::Config = serde_json::from_str(&config_file_contents)?;
 
 	// The create command must run before getting the profile so that configs without profiles can have profiles added to them
 	if let SubCommands::Profile {
@@ -107,30 +90,26 @@ async fn actual_main() -> Result<()> {
 		profile
 	} else {
 		if config.profiles.is_empty() {
-			return Err(Error::Quit (
-				"There are no profiles configured. Add a profile to your config using `ferium profile create`"
-			));
+			bail!("There are no profiles configured. Add a profile to your config using `ferium profile create`")
 		}
 		// Default to first profile if index is set incorrectly
 		config.active_profile = 0;
 		config::write_config(&mut config_file, &config).await?;
-		return Err(Error::Quit(
-			"Active profile index points to a non existent profile. Switched to first profile",
-		));
+		bail!("Active profile index points to a non existent profile. Switched to first profile",)
 	};
 
 	// Run function(s) based on the sub(sub)command to be executed
 	match cli_app.subcommand {
 		SubCommands::AddModrinth { project_id } => {
-			let project = subcommands::add::modrinth(&modrinth, project_id, profile).await?;
+			let project = add::modrinth(&modrinth, project_id, profile).await?;
 			println!("Added {}", project.title);
 		},
 		SubCommands::AddGithub { owner, name } => {
-			let repo = subcommands::add::github(github.repos(owner, name), profile).await?;
+			let repo = add::github(github.repos(owner, name), profile).await?;
 			println!("Added {}", repo.name);
 		},
 		SubCommands::AddCurseforge { project_id } => {
-			let project = subcommands::add::curseforge(&curseforge, project_id, profile).await?;
+			let project = add::curseforge(&curseforge, project_id, profile).await?;
 			println!("Added {}", project.name);
 		},
 		SubCommands::List { verbose } => {
@@ -188,41 +167,69 @@ async fn actual_main() -> Result<()> {
 			// Empty the mods directory
 			let _ = remove_dir_all(&profile.output_dir).await;
 			create_dir_all(&profile.output_dir).await?;
+			let mut error = false;
 			for mod_ in &profile.mods {
 				use libium::config::structs::Mod;
-				if let (Err(err), name) = match mod_ {
+				let result = match mod_ {
 					Mod::CurseForgeProject { name, project_id } => (
-						subcommands::upgrade::curseforge(
+						name,
+						match libium::upgrade::curseforge(
 							&curseforge,
 							profile,
 							*project_id,
 							no_patch_check,
 						)
-						.await,
-						name,
+						.await
+						{
+							Ok(file) => Ok(file.file_name),
+							Err(err) => Err(err),
+						},
 					),
 					Mod::ModrinthProject { name, project_id } => (
-						subcommands::upgrade::modrinth(
+						name,
+						match libium::upgrade::modrinth(
 							&modrinth,
 							profile,
 							project_id,
 							no_patch_check,
 						)
-						.await,
-						name,
+						.await
+						{
+							Ok(version) => Ok(version.files[0].filename.clone()),
+							Err(err) => Err(err),
+						},
 					),
 					Mod::GitHubRepository { name, full_name } => (
-						subcommands::upgrade::github(
+						name,
+						match libium::upgrade::github(
 							&github.repos(&full_name.0, &full_name.1),
 							profile,
 						)
-						.await,
-						name,
+						.await
+						{
+							Ok(asset) => Ok(asset.name),
+							Err(err) => Err(err),
+						},
 					),
-				} {
-					println!("Could not download {} due to {}", name, err);
+				};
+				match result {
+					(name, Ok(file_name)) => {
+						println!(
+							"{} {:40}{}",
+							*TICK,
+							name,
+							format!("({})", file_name).dimmed()
+						);
+					},
+					(name, Err(err)) => {
+						eprintln!("{}", format!("× {:40}{}", name, err).red());
+						error = true;
+					},
 				}
 			}
+			if error {
+				bail!("\nSome mods were not successfully downloaded")
+			}
 		},
 	};
 
@@ -235,8 +242,7 @@ async fn actual_main() -> Result<()> {
 /// Check if `profile` is empty, and if so return an error
 fn check_empty_profile(profile: &config::structs::Profile) -> Result<()> {
 	if profile.mods.is_empty() {
-		Err(Error::EmptyConfigFile)
-	} else {
-		Ok(())
+		bail!("Your currently selected profile is empty! Run `ferium help` to see how to add mods");
 	}
+	Ok(())
 }
